//=========================================================================
//
//	FILE:				CP_PersistantWriter_Binary.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_Persistant writer class for writing our demo documents
//						data as binary data.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 8 $
//						Latest Checkout:  $Date: 2005-08-30 20:31:30 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// CPLAT_Framework
#include "CP_Debugging.h"
#include "CP_File.h"
#include "CP_Persistant.h"
#include "CP_PersistantWriter_Binary.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_PersistantWriter_Binary
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
CP_PersistantWriter_Binary::CP_PersistantWriter_Binary( CPLAT::CP_Persistant *inPersistant )
	: CPLAT::CP_PersistantWriter( inPersistant )
{
}

//=========================================================================
//	FUNCTION:	~CP_PersistantWriter_Binary
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
CP_PersistantWriter_Binary::~CP_PersistantWriter_Binary()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Getters ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	GetType
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Returns the type of data we are writing. In this demo we return 'BIND' for
*	binary data.
*	
*	@exception	none
*
*	@return		FourCharCode		The type of data being written.
*/
CPLAT::FourCharCode CP_PersistantWriter_Binary::GetType()
{
	return( 'BIND' );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Writing ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	Write
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Creates a file for writing into, and then calls the WritePersistantData
*	method to handle the actual writing of the data to the created file. In
*	this demo we will set the files creator code to 'TTXT' to be able to be
*	opened by SimpleText.
*
*	@param		inFileSpec			The CP_FileSystemSpec for the file we
*									are going to create to save the documents
*									data to.
*	@param		inType				The type of file being created.
*	
*	@exception	none
*
*	@return		CP_FileSystemSpec	The file spec of the created file. This file
*									spec points to the file we created, as opposed
*									to the one passed in, which points to the file
*									specification of the file we want to create.
*/
CPLAT::CP_FileSystemSpec CP_PersistantWriter_Binary::Write( const CPLAT::CP_FileSystemSpec& inFileSpec,
	CPLAT::FourCharCode inType )
{
	ASSERT( fPersistant != NULL );
	
	CPLAT::CP_File			dataFile( inFileSpec );
	CPLAT::FourCharCode		creator = CP_FOUR_CHAR_CODE('TTXT');

	dataFile.Create( inType, creator );
	dataFile.Open();

		// call the persistant class to handle writing the actual
		// persistant data to the passsed in file.
	if( fPersistant != NULL )
		fPersistant->WritePersistantData( dataFile, this );

	dataFile.Close();
	
	return( dataFile.GetFileSpec() );
}

#ifdef TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
