//=========================================================================
//
//	FILE:				Undo_ContentView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class that draws rectangles when the user
//						clicks on the view to demonstrate the ability to undo/redo
//						these actions.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 5 $
//						Latest Checkout:  $Date: 2005-08-30 20:34:20 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

#ifndef _H_Undo_ContentView
#define _H_Undo_ContentView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Point.h"
#include "CP_UndoAction.h"
#include "CP_View.h"

/*!
*	\class	Undo_ContentView
*	\brief	CP_View class that draws rectangles when the user clicks on the view to
*			demonstrate the ability to undo/redo these actions.
*/

class CP_EXPORT Undo_ContentView : public CPLAT::CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										Undo_ContentView(
												CPLAT::CP_View *inSuperview,
												const CPLAT::CP_Point& inUpperLeft,
												const CPLAT::CP_Size& inContentSize );
	//! Destructor.
	virtual								~Undo_ContentView() throw();
	//@}
		
protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CPLAT::CP_Canvas& inCanvas,
												CPLAT::CP_Region& inUpdateArea );
	//@}

	//! @name Events
	//@{
	virtual	void						InitEventHandler( CPLAT::CP_EventHandler& inEventHandler );

			void 						HandleControlHitEvent( CPLAT::CP_Event_Base& inEvent );
	//@}

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

class CP_EXPORT CreateViewAction : public CPLAT::CP_UndoAction
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CreateViewAction(
												CPLAT::CP_String& inUndoActionText,
												CPLAT::CP_Point& inMouseLocation,
												CPLAT::CP_View *inSuperview );
	//! Destructor.
										~CreateViewAction() throw();
	//@}
						
	void 	Do();
	
protected:
	//! @name Undo / Redo
	//@{
	virtual	bool						UndoSelf();
	virtual	bool						RedoSelf();
	//@}


	// class data
protected:
	CPLAT::CP_Point						fMouseLocation;
	CPLAT::CP_View						*fSuperview;
	
	CPLAT::CP_View						*fCreatedView;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#endif	// _H_Undo_ContentView

