//=========================================================================
//
//	FILE:				CP_ARGBColor.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Color representation using 4 float values for red, green, blue and alpha components.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				On the Mac a color is composed of 3 floats for the individual
//						compoents. While Windows is composed of 3 chars (max 255).
//						For the Windows version of the color we return the high
//						byte of our float.
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_ARGBColor
#define _H_CP_ARGBColor
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Object.h"
#include "CP_RGBColor.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_ARGBColor
*	@brief		Color representation using 4 float values for red, green, blue and alpha components.
*
*				NOTE: On the Mac a color is composed of 3 floats for the individual
*				compoents. While Windows is composed of 3 chars (max 255). For the Windows
*				version of the color we return the high byte of our float.
*/
class CP_EXPORT CP_ARGBColor
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ARGBColor();
										CP_ARGBColor(
											float inRed,
											float inGreen,
											float inBlue,
											float inAlpha );
										CP_ARGBColor( const CP_ARGBColor& inColor );
										CP_ARGBColor( const CP_RGBColor& inColor );
	//! Destructor.
	virtual								~CP_ARGBColor();
	//@}

	//! @name Components
	//@{
			float						red() const;
			float						green() const;
			float						blue() const;
			float						alpha() const;
	//@}
			
	//! @name Getters
	//@{
			CP_RGBColor					GetRGBColor();
			CP_RGBColor					GetRGBColor() const;
	//@}
			
	//! @name Operators
	//@{
	bool								operator == ( const CP_ARGBColor& inColor ) const;
	bool								operator != ( const CP_ARGBColor& inColor ) const;

	CP_ARGBColor&						operator = ( const CP_ARGBColor& inColor );
	CP_ARGBColor&						operator = ( const CP_RGBColor& inColor );
	//@}

	//! @name Setters
	//@{
			void						Set(
												float inRed,
												float inGreen,
												float inBlue,
												float inAlpha );

										/*!
										*	Sets the alpha component of the color.
										*	
										*	@param		inAlpha				The alpha component of the color.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetAlpha( float inAlpha )					{ fAlpha = inAlpha; }
			
										/*!
										*	Sets the color values from the passed in CP_ARGBColor.
										*	
										*	@param		inColor				CP_ARGBColor to set the colors from.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						Set( const CP_ARGBColor& inColor )			{ operator = ( inColor ); }

										/*!
										*	Sets the color values from the passed in CP_RGBColor.
										*	
										*	@param		inColor				CP_RGBColor to set the colors from.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						Set( const CP_RGBColor& inColor )			{ operator = ( inColor ); }
	//@}

	// class data
protected:
	float								fRed;		/*! The red color component. */
	float								fGreen;		/*! The green color component. */
	float								fBlue;		/*! The blue color component. */
	float								fAlpha;		/*! The alpha color component. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_RGBColor


