//=========================================================================
//
//	FILE:				CP_AutoOSPtr.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A smart pointer like class for dealing with OSPtrs.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_AutoOSPtr
#define _H_CP_AutoOSPtr
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

	// Stanndard Library
#include <new>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_AutoOSPtr
*	@brief		A smart pointer like class for dealing with Ptrs.
*	
*				This class wraps an OSs Ptr object into a smart pointer that manages
*				the allocation and deallocation automatically.
*/
class CP_EXPORT CP_AutoOSPtr
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AutoOSPtr();
										CP_AutoOSPtr( CP_AutoOSPtr& inAutoPtr );
	explicit							CP_AutoOSPtr( OSPtr inPtr );
	explicit							CP_AutoOSPtr( size_t inSize );
	//! Destructor.
										~CP_AutoOSPtr() throw();
	//@}
	
	//! @name operators
	//@{
			CP_AutoOSPtr&				operator = (CP_AutoOSPtr& ioPtr) throw();

										/*!
										*	Returns the Ptr as an OSPtr.
										*	
										*	@exception	none
										*	
										*	@return		OSPtr		The platforms OS Ptr.
										*/
										operator OSPtr () const throw()						{ return( fPtr ); }
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the Ptr as an OSPtr.
										*	
										*	@exception	none
										*	
										*	@return		OSPtr		The platforms OS Ptr.
										*/
			OSPtr						Get() const throw()									{ return( fPtr ); }
	//@}

	//! @name Management
	//@{
			OSPtr						Release() throw();

			void						Reset( OSPtr inPtr = NULL ) throw();
	//@}

protected:

	// class data
protected:
	OSPtr								fPtr;		/*! Platforms Ptr object.  */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

/*!
*	@class		CP_AutoOSPtrTemp
*	@brief		A smart pointer like class for dealing with Ptrs.
*	
*				This class wraps an OSs Ptr object into a smart pointer that manages
*				the allocation and deallocation automatically.
*/
template <typename T>
class CP_EXPORT CP_AutoOSPtrTemp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AutoOSPtrTemp();
										CP_AutoOSPtrTemp( CP_AutoOSPtrTemp& inAutoPtr );
	explicit							CP_AutoOSPtrTemp( T *inPtr );
	//! Destructor.
										~CP_AutoOSPtrTemp() throw();
	//@}
	
	//! @name operators
	//@{
			CP_AutoOSPtrTemp&			operator = (CP_AutoOSPtrTemp& ioPtr) throw();
	
										/*!
										*	Returns the Ptr as an OSPtr.
										*	
										*	@exception	none
										*	
										*	@return		T		The platforms OS Ptr.
										*/
										operator T () const throw()							{ return( fPtr ); }
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the Ptr as an OSPtr.
										*	
										*	@exception	none
										*	
										*	@return		T		The platforms OS Ptr.
										*/
			T							Get() const throw()									{ return( fPtr ); }
	//@}

	//! @name Management
	//@{
			T							Release() throw();

			void						Reset( T inPtr = NULL ) throw();
	//@}

protected:

	// class data
protected:
	T									*fPtr;		/*! T ptr object.  */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_AutoOSPtr

