//=========================================================================
//
//	FILE:				CP_M_Broadcaster.h
//	
//						Copyright  2003. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Mix-in class for objects that can broadcast messages
//						to CP_M_Listeners.
//	
//	HISTORY:			1st Version - November 2003.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Revision: 191 $
//						Latest Checkout:  $Date: 2005-05-23 14:36:50 -0700 (Mon, 23 May 2005) $
//
//=========================================================================

#ifndef _H_CP_M_Broadcaster
#define _H_CP_M_Broadcaster
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Message.h"

	// Standard Library
#include <list>

CPLAT_Begin_Namespace_CPLAT

	// Forward declaration
class CP_EXPORT CP_M_Listener;

/*! \fn typedef CP_Listeners
 *  An STL list typedef to hold all registered CP_M_Listener objects registered with this class. 
 */
#ifndef TARGET_OS_LINUX
	typedef std::list<CP_M_Listener *, std::allocator<CP_M_Listener *> >						CP_Listeners;
	typedef std::list<CP_M_Listener *, std::allocator<CP_M_Listener *> >::iterator				CP_Listeners_Iterator;
	typedef std::list<CP_M_Listener *, std::allocator<CP_M_Listener *> >::reverse_iterator		CP_Listeners_ReverseIterator;
#else
	typedef std::list<CP_M_Listener * >															CP_Listeners;
	typedef std::list<CP_M_Listener * >::iterator												CP_Listeners_Iterator;
	typedef std::list<CP_M_Listener * >::reverse_iterator										CP_Listeners_ReverseIterator;
#endif

/*!
*	@class		CP_M_Broadcaster
*	@brief		Mix-in class for objects that can broadcast messages to CP_M_Listeners
*/
class CP_EXPORT CP_M_Broadcaster
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
	virtual								~CP_M_Broadcaster();
	//@}
			
	//! @name Listeners
	//@{
			void						AddListener( CP_M_Listener *inListener );
			void						RemoveListener( CP_M_Listener *inListener );
	//@}

	//! @name Message
	//@{
			void						BroadcastMessage( CP_Message& inMessage );
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_M_Broadcaster();
	//@}

private:
	//! @name No Copy
	//@{
										CP_M_Broadcaster( const CP_M_Broadcaster& inBroadcaster );
			CP_M_Broadcaster&			operator = ( const CP_M_Broadcaster& inString );
	//@}

	// class data
protected:
	CP_Listeners						*fListeners;	/*! A list of the listeners for this object. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_M_Broadcaster

