//=========================================================================
//
//	FILE:				CP_M_DeferredTask.h
//	
//						Copyright  2005. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Mix-in class for objects that need to do a one time task
//						at the next application event loop iteration. Typically these
//						will be objects that need to delete themselves, but need to
//						wait because they are in the middle of a routine.
//	
//	HISTORY:			1st Version - May 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 209 $
//						Latest Checkout:  $Date: 2005-06-10 17:22:36 -0700 (Fri, 10 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_M_DeferredTask
#define _H_CP_M_DeferredTask
#ifndef __BUILD_LINUX__
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_M_DeferredTask
*	@brief		Mix-in class for objects that need to do a one time task at the next application
*				event loop iteration. Typically these will be objects that need to delete
*				themselves, but need to wait because they are in the middle of a routine.
*/
class CP_EXPORT CP_M_DeferredTask
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_M_DeferredTask();
	//! Destructor.
	virtual								~CP_M_DeferredTask();
	//@}

	//! @name Task
	//@{
			void						DoTask();
	//@}

protected:
	//! @name Task
	//@{
										/*!
										*	Subclasses override to handle the specific task when called.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual	void						OnDoTask()							{}
	//@}

protected:

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

/*!
*	@class		CP_DeferredViewDeleter
*	@brief		A CP_M_DeferredTask to handle deleting CP_Views at the next idle loop time.
*				This class would typically be used if you have a UI element that will
*				cause a some other subpane to be deleted, but can't do it immediately.
*				For example: a button in a window to delete the owner window.
*/
class CP_EXPORT CP_DeferredViewDeleter : public CP_M_DeferredTask
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DeferredViewDeleter( CP_View *inView );
	//! Destructor.
	virtual								~CP_DeferredViewDeleter();
	//@}

protected:
	//! @name Task
	//@{
	virtual	void						OnDoTask();
	//@}

protected:
	CP_View								*fViewToDelete;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_M_DeferredTask



