//=========================================================================
//
//	FILE:				CP_PropertyList_Node.h
//	
//						Created by Jens Miltner on 01.12.04.
//	
//	PURPOSE:			CP_PropertyList nodes.
//	
//	HISTORY:			1st Version - January 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 144 $
//						Latest Checkout:  $Date: 2005-04-12 21:56:34 -0700 (Tue, 12 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_PropertyList_Node
#define _H_CP_PropertyList_Node

#include "CP_PropertyList_Types.h"
#include "CP_M_ReferenceCounted.h"
#include "CP_New.h"
#include <map>
#include "CP_XMLDocument.h"

CPLAT_Begin_Namespace_CPLAT

class CP_InStream;
class CP_OutStream;
class CP_LogFile;

namespace CP_PropertyList {

	#if CP_PropertyList_Uses_CoreFoundation
		template <class Class_t, class CF_t> class NodeFactoryImp;
	#endif
	
	/*!
	 Base class for all property list nodes.
	*/
	class CP_EXPORT Node : public CP_M_RefCounted
	{
	public:
		virtual ~Node() throw();
		virtual NodeType_t NodeType() const throw() = 0;
		virtual void WriteXMLData(CP_OutStream& inXMLDataStream) const;
		static Node* FromXMLData(CP_InStream& inXMLDataStream, NodeType_t inExpectedNodeType=NodeType_Any);

		virtual void PrintToLog(CP_LogFile& inLogFile, SInt32 inLevel=0, SInt32 inIndentLevel = 0) const = 0;
	#if CP_Debug	
		void Dump() const;
	#endif
		 
	#if CP_PropertyList_Uses_CoreFoundation
		virtual CFTypeRef GetNodeCFRef() const = 0;
	#endif

	#if CP_PropertyList_Uses_CoreFoundation
		static Node* CreateNodeForCFObject(CFTypeRef inObject);
	#endif

	#if !CP_PropertyList_Uses_CoreFoundation
		//! write XML data for this node only (i.e. without xml header, plist root, etc.)
		virtual void WriteNodeXMLData(CP_OutStream& inXMLDataStream, int inLeadingSpaces) const = 0;
	#endif
	};

	#if CP_PropertyList_Uses_CoreFoundation
		class NodeFactory
		{
		public:
			static Node* CreateNodeForCFObject(CFTypeRef inCFObject);
			virtual ~NodeFactory() {}
		protected:
			static std::map<CFTypeID, NodeFactory*>	*gFactories;
			void RegisterFactory(CFTypeID inTypeID, NodeFactory* factory);
			void UnregisterFactory(CFTypeID inTypeID);
			virtual Node* operator()(CFTypeRef inCFObject) = 0;
		};
		template <class Class_t, class CF_t>
		class NodeFactoryImp : public NodeFactory
		{
			CFTypeID		fTypeID;
		public:
			NodeFactoryImp(CFTypeID inTypeID) : fTypeID(inTypeID) { RegisterFactory(inTypeID, this); }
			~NodeFactoryImp() { UnregisterFactory(fTypeID); }
		protected:
			virtual Node* operator()(CFTypeRef inCFObject);
		};
		
		template <class Class_t, class CF_t> 
		Node* NodeFactoryImp<Class_t, CF_t>::operator()(CFTypeRef inCFObject) 
		{ 
			ASSERT(CFGetTypeID(inCFObject) == fTypeID);
			return CP_new Class_t(static_cast<CF_t>(inCFObject)); 
		}
	#endif // CP_PropertyList_Uses_CoreFoundation
} // end of namespace CP_PropertyList

CPLAT_End_Namespace_CPLAT

#endif // _H_CP_PropertyList_Node
