/*
=========================================================================

	FILE:		CP_Registry.h
	
				Copyright  2004. Ken Stahlman. All Rights Reserved.
	
	PURPOSE:	The CP_Registry class is used under Windows only to read
				and write data to the Windows registry.
	
	HISTORY:	1st Version - June 2003.
	
	NOTES:		This file is for Windows only.

	VERSION:	Current Revision: $Rev: 106 $
				Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $

=========================================================================
*/
#ifndef _H_CP_Registry
#define _H_CP_Registry
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_WIN32

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_Size.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_RGBColor.h"
#include "CP_Size.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

class CP_EXPORT CP_Registry
{
public:
									// Construction / destruction
								CP_Registry( HKEY inHKeyRoot = HKEY_LOCAL_MACHINE );
	virtual						~CP_Registry();

									// Getters
			bool				GetValue(
										const CP_String& inKey,
										SInt16 *outValue );
			bool				GetValue(
										const CP_String& inKey,
										UInt16 *outValue );
			bool				GetValue(
										const CP_String& inKey,
										SInt32 *outValue );
			bool				GetValue(
										const CP_String& inKey,
										UInt32 *outValue );
			bool				GetValue(
										const CP_String& inKey,
										float *outValue );
			bool				GetValue(
										const CP_String& inKey,
										double *outValue );
			bool				GetValue(
										const CP_String& inKey,
										CP_String& outValue );
			bool				GetValue(
										const CP_String& inKey,
										CP_Point& outValue );
			bool				GetValue(
										const CP_String& inKey,
										CP_Size& outValue );
			bool				GetValue(
										const CP_String& inKey,
										CP_Rect& outValue );
			bool				GetValue(
										const CP_String& inKey,
										CP_RGBColor& outValue );
			bool				GetValue(
										const CP_String& inKey,
										CP_FileSystemSpec& outValue );
			bool				GetValue(
										const CP_String& inKey,
										UInt8 *outBuffer, UInt32 inBufferSize );										

									// Keys
			bool				VerifyKey(
										HKEY inHKeyRoot,
										const CP_String& inPath );
			bool				VerifyKey( const CP_String& inPath );
			bool				VerifyValue( const CP_String& inValue );
			bool				CreateKey(
										HKEY inHKeyRoot,
										const CP_String& inPath );
			bool				DeleteValue( const CP_String& inValue );
			bool				DeleteValueKey( 
										HKEY inHKeyRoot,
										const CP_String& inPath );

									// Registry
			bool				Open(
										HKEY inHKeyRoot,
										const CP_String& inPath );
			void				Close();

									// Setters
			bool				SetValue(
										const CP_String& inKey,
										SInt16 inValue );
			bool				SetValue(
										const CP_String& inKey,
										UInt16 inValue );
			bool				SetValue(
										const CP_String& inKey,
										SInt32 inValue );
			bool				SetValue(
										const CP_String& inKey,
										UInt32 inValue );
			bool				SetValue(
										const CP_String& inKey,
										float inValue );
			bool				SetValue(
										const CP_String& inKey,
										double inValue );
			bool				SetValue(
										const CP_String& inKey,
										const CP_String& inValue );
			bool				SetValue(
										const CP_String& inKey,
										const CP_Point& inPoint );
			bool				SetValue(
										const CP_String& inKey,
										const CP_Size& inSize );
			bool				SetValue(
										const CP_String& inKey,
										const CP_Rect& inRect );
			bool				SetValue(
										const CP_String& inKey,
										const CP_RGBColor& inRGBColor );
			bool				SetValue(
										const CP_String& inKey,
										const CP_FileSystemSpec& inFileSpec );
			bool				SetValue(
										const CP_String& inKey,
										const UInt8 *inData, UInt32 inDataSize );

protected:

	// class data
protected:
	HKEY					fHKey;
	CP_String				fPath;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif

#endif	// _H_CP_Registry

