//=========================================================================
//
//	FILE:				CP_SMTP.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to implement the SMTP protocol.
//
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_SMTP
#define _H_CP_SMTP
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_MailMessage.h"
#include "CP_NetProtoBase.h"
#include "CP_Socket.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

	// SMTP commands
#define kCP_SMTP_CMD__User				"USER"
#define kCP_SMTP_CMD__Hello				"HELO"
#define kCP_SMTP_CMD__MailFrom			"MAIL FROM"
#define kCP_SMTP_CMD__Recipient			"RCPT TO"
#define kCP_SMTP_CMD__Data				"DATA"
#define kCP_SMTP_CMD__Reset				"RSET"
#define kCP_SMTP_CMD__Quit				"QUIT"

const UInt16 kCP_MAX_SMTP_LINE_LENGTH	= 1000;

/*!
*	@class		CP_SMTP
*	@brief		A CP_NetProtoBase class that implements the SMTP protocol for sending messages.
*/
class CP_EXPORT CP_SMTP : public CP_NetProtoBase
{
public:
		// default SMTP port number
	enum { kCP_SMTPPortNumber = 25 };

	enum SMTPConnectState
	{
		kCP_SMTPClosed,
		kCP_SMTPConnected
	};

	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_SMTP( CP_Socket *inSocketConnection );
	//! Destructor.
	virtual								~CP_SMTP() throw();
	//@}

	//! @name Connection
	//@{
				void					Connect(
												const CP_String& inAddress,
												SInt16 inPortNumber = CP_SMTP::kCP_SMTPPortNumber );
												
				void					WaitForServerID();
	//@}

	//! @name Commands
	//@{
				void					SendDATA();
				void					SendHELO( const CP_String& inAddress );
				void					SendQUIT();
				void					SendRSET();
				void					SendMAIL( const CP_String& inFrom );
				void					SendRCPT( const CP_String& inTo );
				void					SendRCPT( CP_Strings_Array *inList );
	//@}

	//! @name Sending
	//@{
				void					SendOneMessage( CP_MailMessage& inMailMessage );
				void					SendMessageData( CP_MailMessage& inMailMessage );

				void					SendSMTPEncodeData( const CP_String& inMessageData );
	//@}

protected:
	
	// class data
protected:
	SMTPConnectState					fConnectState;			/*! The current state of the connection with the SMTP server. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_SMTP


