//=========================================================================
//
//	FILE:				CP_WindowMenuManager.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Singleton class to handle the windows menu for OSs that don't support natively.
//
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 182 $
//						Latest Checkout:  $Date: 2005-05-20 14:28:56 -0700 (Fri, 20 May 2005) $
//
//=========================================================================
#ifndef _H_CP_WindowMenuManager
#define _H_CP_WindowMenuManager
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_NoCopy.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_WindowMenuManager
*	@brief		Singleton class to handle the windows menu for OSs that don't support natively.
*/
class CP_EXPORT CP_WindowMenuManager : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_WindowMenuManager();
										CP_WindowMenuManager( SInt16 inBeforeMenuID );
	//! Destructor.
	virtual								~CP_WindowMenuManager() throw();
	//@}
			
	//! @name Instance
	//@{
	static	CP_WindowMenuManager		*Instance();
	//@}

protected:

	// class data
protected:
	static CP_WindowMenuManager			*fsInstance;

	OSMenu								fOSMenu;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
			void						InitPlatformMenu( SInt16 inBeforeMenuID = 0 );
#endif

#if TARGET_OS_WIN32
private:
			void						InitPlatformMenu( SInt16 inBeforeMenuID = 0 );
#endif

#if TARGET_OS_LINUX
private:
			void						InitPlatformMenu( SInt16 inBeforeMenuID = 0 );
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_WindowMenuManager

