//=========================================================================
//
//	FILE:				CP_PutFileDialog.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Presents the platforms PutFileDialog to ask user where
//						they want a file to be saved to.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_PutFileDialog
#define _H_CP_PutFileDialog
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_FileSystemDialogsReply.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_PutFileDialog
*	@brief		Presents the platforms PutFileDialog.
*/
class CP_EXPORT CP_PutFileDialog : public CP_FileSystemDialogs_Reply
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_PutFileDialog();
	//! Destructor.
	virtual								~CP_PutFileDialog() throw();
	//@}

	//! @name Getters
	//@{
			CP_FileSystemSpec			GetFileSpec()										{ return( fSelectedFileSpec ); }
	//@}

	//! @name Setters
	//@{
			void						SetCreator( FourCharCode inCreator )				{ fDefaultCreator = inCreator; }
			void						SetFileType( FourCharCode inFileType )				{ fDefaultFileType = inFileType; }
	//@}

protected:
	//! @name Events
	//@{
	virtual	void						HandleUserAction(
												CP_FileSystemDialogs_Base *inSendingDialog,
												CP_E_NavDialogResult inNavDialogResult );
	//@}

	// class data
protected:
	CP_FileSystemSpec					fSelectedFileSpec;		/*! File to be used in selecting location. */
	
	FourCharCode						fDefaultCreator;		/*! Creator code for the file. */
	FourCharCode						fDefaultFileType;		/*! File type for the file. */
			
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
protected:
	//! @name Dialog handlers
	//@{
	virtual	NavDialogRef				CreateDialog(
												NavDialogCreationOptions& inOptions, 
												NavEventUPP inEventUPP,
												void *inUserData );

	virtual	void						ProcessReply();
	//@}
#endif

#if TARGET_OS_WIN32
protected:
	//! @name Dialog handlers
	//@{
			void						RunPlatformDialog();

	static UINT APIENTRY				CP_PutFileHook(
												HWND hWnd,
												UINT uMsg,
												WPARAM wParam,
												LPARAM lParam );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_PutFileDialog

