//=========================================================================
//
//	FILE:				CP_XMLWriter.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to write our data in XML format.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_XMLWriter
#define _H_CP_XMLWriter
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_File.h"
#include "CP_NoCopy.h"
#include "CP_OutStream.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_EXPORT CP_XMLWriter_Element;

/*!
*	@class		CP_XMLWriter
*	@brief		A class for writing out data in XML format.
*
*	@ingroup	XML
*	
*/
class CP_EXPORT CP_XMLWriter : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_XMLWriter(
												CP_OutStream *inOutStream,
												const CP_String& inXMLVersion = CP_String("1.0"),
												const CP_String& inXMLEncoding = CP_String("UTF-8") );
	//! Destructor.
	virtual								~CP_XMLWriter();
	//@}
	
	//! @name Getters
	//@{
			CP_OutStream				*GetOutputStream()							{ return( fOutStream ); }
	//@}

	//! @name Elements
	//@{
			void						SetRootElement( CP_XMLWriter_Element *inRootElement );
	//@}
												
	//! @name Output
	//@{
			bool						WriteAll();

	virtual void 						WriteBytes(
												const void *inBuffer,
												UInt32 inNumberBytes );

	virtual	void						WriteHeader();
	virtual	void						WriteTrailer();
	//@}

	// class data members
protected:
	CP_OutStream						*fOutStream;		/*! The CP_OutStream object that we will write the data to.
																This can be either a CP_HandleBlock or CP_File out stream. */

	CP_String							fXMLVersion;		/*! The version of the XML data -- currently should be 1.0. */
    								
	CP_String							fXMLEncoding;		/*! The encoding that the XML data is written in. */

	CP_XMLWriter_Element				*fRootElement;		/*! The root element for the XML data. */
    								
	bool								fRootCreated;		/*! Flag to indicate if the root element has been created. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_XMLWriter


