//=========================================================================
//
//	FILE:				CP_Menu_Imp_Gtk.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			MacOS implementation of the CP_Menu class.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 4 $
//						Latest Checkout:  $Date: 2004-12-22 19:17:46 -0800 (Wed, 22 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Menu_Imp_Gtk
#define _H_CP_Menu_Imp_Gtk
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_LINUX

	// CPLAT_Framework
#include "CP_CommandNos.h"
#include "CP_FrameworkStructures_Linux.h"
#include "CP_NoCopy.h"
#include "CP_String.h"
#include "CP_Types.h"

	// Standard Library
#include <vector>

	// Gtk
#include <gtk/gtk.h>

CPLAT_Begin_Namespace_CPLAT

	// forward class declaration
class CP_Menu_Imp;

//=========================================================================
//	CP_CommandsNoArray
//=========================================================================
/*!
*	Standard library vector typedef to hold a list of CP_CommandNos for the menu
*	items in this menu. There is 1 item for each item in the menu.
*/
typedef std::vector<CP_CommandNo>															CP_CommandsNo_Array;
typedef std::vector<CP_CommandNo>::iterator													CP_CommandsNo_Iterator;
typedef std::vector<CP_CommandNo>::reverse_iterator											CP_CommandsNo_ReverseIterator;

	// structure to hold info about a single menu item in our menu.
	// this structure is passed to the menu item callback routine when a user
	// selects a menu item
struct CP_S_MenuItem_Linux
{
	CP_Menu_Imp		*fOwnerMenu;			// pointer to ourselves -- CP_Menu we are a menu item in
	char			fMenuItemText[ 256 ];	// text to the menu item
	SInt16			fMenuItemIndex;			// index into list of menu item structures
	GtkWidget		*fMenuItemWidget;		// Gtk menu item widget
};

typedef std::vector<CP_S_MenuItem_Linux * >													CP_MenuItems_Array;
typedef std::vector<CP_S_MenuItem_Linux * >::iterator										CP_MenuItems_Iterator;
typedef std::vector<CP_S_MenuItem_Linux * >::reverse_iterator								CP_MenuItems_ReverseIterator;

/*!
*	@class	CP_Menu_Imp
*	@brief	Linux / Gtk implementation of the CP_Menu class.
*/
class CP_EXPORT CP_Menu_Imp : CP_NoCopy
{
public:
									// Construction / destruction
										CP_Menu_Imp();
										CP_Menu_Imp(
												SInt16 inMenuID,
												const CP_String& inMenuTitle );
										CP_Menu_Imp( const CP_String& inMenuTitle );
										CP_Menu_Imp(
												SInt16 inMenuID,
												const CP_String& inMenuTitle,
												OSMenu inOSMenu );
	virtual								~CP_Menu_Imp() throw();
						
									// Commands
	virtual	CP_CommandNo				GetCommandNo( SInt16 inMenuItemIndex );

											// Getters
										/*!
										*	Return the MacOS platform menu reference.
										*	
										*	@return	OSMenu		MacOS menu reference.
										*/
			OSMenu						GetOSMenu()											{ return( fOSMenu ); }
			OSMenu						GetMenuItems()										{ return( fMenuItems ); }

										/*!
										*	Return the ID of this menu.
										*
										*	@exception	none
										*	
										*	@return		SInt16		ID of this menu.
										*/
			SInt16						GetID()												{ return( fMenuID ); }

			UInt16						GetMenuItemCount();

			CP_String					GetItemText( SInt16 inMenuItemIndex );

									// Menu Items
			void						AppendItem(
												const CP_String& inMenuItem,
												CP_CommandNo inCommand = CP_CMD_NoCommand );
			void						AppendSeparator();
			void						DeleteItem( UInt16 inItemIndex );

			CP_MenuItems_Array			*GetMenuItemsArray()								{ return( fMenuItemStructs_Linux ); }

	//! @name Setters
	//@{
			void						SetTitle( const CP_String& inTitle );
	//@}

protected:
			void						LoadOSMenu(
												SInt16 inMenuID,
												bool inPopup );

private:
									// Callbacks
	static	void						Selection_callback(
												GtkWidget *inWidget,
												gpointer inData );
	static	gboolean					Clicked_callback(
												GtkWidget *inWidget,
												GdkEvent *inEvent,
												CP_Menu_Imp *inMenu );

									// Commands
			void						BuildCommandTable();

									// Menu items
			void						AddMenuItem_private( CP_S_MenuItem_Linux *inMenuItem );
			CP_S_MenuItem_Linux 		*GetMenuItem_Private( SInt16 inIndex ) const;

			void 						LoadMenu_LinuxStruct(
												CP_S_MenuItems_Linux *inMenuItems,
												const CP_String& inMenuTitle );

	// class data
protected:
	SInt16								fMenuID;		/*! Menu id of this menu. */
	OSMenu								fOSMenu;		/*! MacOS menu reference. */

	CP_CommandsNo_Array					*fCommands;		/*! Table of commands for each menu item. */

	OSMenu								fMenuItems;

								/*! List of structures we maintain about each item in the menu, used to map between CPLAT
								information for a menu, and the GtkWidget */
	CP_MenuItems_Array					*fMenuItemStructs_Linux;
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_LINUX

#endif	// _H_CP_Menu_Imp_Gtk

