//=========================================================================
//
//	FILE:				CP_Resource_Imp_Linux.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A singleton class that manages resources entries used
//						with the CP_Resource class under Linux.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Resource_Imp_Linux
#define _H_CP_Resource_Imp_Linux
#if ! TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_LINUX

	// CPLAT_Framework
#include "CP_OSTypes.h"
#include "CP_Types.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

	// Used by Linux builds to handle resources
struct CP_S_ResourceEntry
{
	SInt16						fResourceID;		/*! ID of the resource. */
	FourCharCode				fResourceType;		/*! Type of resource. */
	OSHandle					fResourceData;		/*! Resource data */

	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_S_ResourceEntry();
								CP_S_ResourceEntry(
										FourCharCode inResourceType,
										SInt16 inResourceID,
										OSHandle inResourceData );
								CP_S_ResourceEntry( const CP_S_ResourceEntry& inResourceEntry );
	//@}

	//! @name operators
	//@{
			CP_S_ResourceEntry&	operator =( const CP_S_ResourceEntry& inResourceEntry );
	bool						operator ==( const CP_S_ResourceEntry& inResourceEntry );
	//@}
};


typedef std::vector<CP_S_ResourceEntry >															CP_Resource_Array;
typedef std::vector<CP_S_ResourceEntry >::iterator													CP_Resource_Iterator;
typedef std::vector<CP_S_ResourceEntry >::reverse_iterator											CP_Resource_ReverseIterator;

class CP_EXPORT CP_Resource_Imp_Linux
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Resource_Imp_Linux();					
	//! Destructor.
	virtual								~CP_Resource_Imp_Linux();
	//@}

	//! @name Instance
	//@{
	static	CP_Resource_Imp_Linux		*Instance();
	//@}

	//! @name Resource Data
	//@{
			void						AddEntry(
												FourCharCode inResType,
												SInt16 inResID,
												OSHandle inResData );
												
			OSHandle					GetResourceData(
												FourCharCode inResType,
												SInt16 inResID );									
	//@}
												
	// class data
protected:
	static CP_Resource_Imp_Linux		*fsInstance;		/*! Single instance of this class. */
	
	CP_Resource_Array					fResourcces;		/*! List of resource entries. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#define CP_ADD_RESOURCE_ENTRY(resType, resID, resData)				{ if( CPLAT::CP_Resource_Imp_Linux::Instance() != NULL ) CPLAT::CP_Resource_Imp_Linux::Instance()->AddEntry( resType, resID, resData ); }

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_LINUX

#endif	// _H_Resource_Imp_Linux

