//=========================================================================
//
//	FILE:				CP_CarbonEvent_Params.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Definitions for various CarbonEvent parameter types.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 229 $
//						Latest Checkout:  $Date: 2005-07-05 22:10:17 -0700 (Tue, 05 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_CarbonEvent_Params
#define _H_CP_CarbonEvent_Params
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Exceptions.h"
#include "CP_TypeTraits.h"

	// Standard Library
#include <new>

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC
	struct CP_CarbonEvent_ParamTypeTrait_Base
	{
		static bool					HasParameter(
											EventRef inEvent, 
											EventParamName inName, 
											EventParamType inType ) throw();
	};

	template <EventParamType TYPE> struct CP_CarbonEvent_ParamTypeTrait : public CP_CarbonEvent_ParamTypeTrait_Base
	{
		typedef typename CP_TypeTrait<TYPE>::Type	Type;

										// Getters
		static OSStatus				Get(
											EventRef inEvent, 
											EventParamName inName, 
											Type& inOutValue) throw();

										// Setters
		static OSStatus				Set(
											EventRef inEvent, 
											EventParamName inName, 
											const Type& inValue) throw();
	};

	template <EventParamName NAME, EventParamType TYPE> struct CP_CarbonEvent_Param
	{
		typedef CP_CarbonEvent_ParamTypeTrait<TYPE>		IOType;
		typedef typename IOType::Type					ValueType;

										// Getters
		static ValueType			Get( EventRef inEvent );
		static void					Get(
											EventRef inEvent,
											ValueType& inValue );
		static bool					Get(
											EventRef inEvent,
											ValueType& inValue,
											const std::nothrow_t& ) throw();

										// Setters
		static void					Set(
											EventRef inEvent,
											const ValueType& inValue );

										// Testers
		static bool					HasParameter( EventRef inEvent ) throw();
	};

	template <EventParamType TYPE> struct CP_CarbonEvent_DirectParam 
		: public CP_CarbonEvent_Param<kEventParamDirectObject, TYPE>
	{
	};

	#if TARGET_CC_METROWERKS
	#pragma mark === CP_CarbonEvent_ParamTypeTrait_Base ==
	#endif

	//=========================================================================
	//	FUNCTION:	Get
	//	
	//	VISIBLITY:	Public
	//=========================================================================
	template <EventParamType TYPE>
	OSStatus CP_CarbonEvent_ParamTypeTrait<TYPE>::Get( EventRef inEvent, EventParamName inName, Type& inOutValue ) throw()
	{
		OSStatus		status = CP_noErr;
		
		status = ::GetEventParameter( inEvent, inName, TYPE, NULL, sizeof(inOutValue), NULL, &inOutValue );
		
		return( status );
	}

	//=========================================================================
	//	FUNCTION:	Get
	//	
	//	VISIBLITY:	Public
	//=========================================================================
	template <EventParamType TYPE>
	OSStatus CP_CarbonEvent_ParamTypeTrait<TYPE>::Set( EventRef inEvent, 
		EventParamName inName, const Type& inValue ) throw()
	{
		OSStatus		status = CP_noErr;
		
		status = ::SetEventParameter( inEvent, inName, TYPE, sizeof(inValue), &inValue );
		
		return( status );
	}

	#if TARGET_CC_METROWERKS
	#pragma mark === CP_CarbonEvent_Param ==
	#endif

	//=========================================================================
	//	FUNCTION:	HasParameter
	//	
	//	VISIBLITY:	Public
	//=========================================================================
	template <EventParamName NAME, EventParamType TYPE>
	bool CP_CarbonEvent_Param<NAME, TYPE>::HasParameter( EventRef inEvent ) throw()
	{
		return( IOType::HasParameter( inEvent, NAME, TYPE ) );
	}

	//=========================================================================
	//	FUNCTION:	Get
	//	
	//	VISIBLITY:	Public
	//=========================================================================
	template <EventParamName NAME, EventParamType TYPE>
	typename CP_CarbonEvent_Param<NAME, TYPE>::ValueType
	CP_CarbonEvent_Param<NAME, TYPE>::Get( EventRef inEvent )
	{
		OSStatus			status = CP_noErr;
		ValueType			value;
		
		status = IOType::Get( inEvent, NAME, value );
		CP_ThrowIfOSErr(status);
		
		return( value );
	}

	//=========================================================================
	//	FUNCTION:	Get
	//	
	//	VISIBLITY:	Public
	//=========================================================================
	template <EventParamName NAME, EventParamType TYPE>
	void CP_CarbonEvent_Param<NAME, TYPE>::Get( EventRef inEvent, ValueType& value )
	{
		OSStatus			status = CP_noErr;
		
		status = IOType::Get( inEvent, NAME, value );
		CP_ThrowIfOSErr(status);
	}

	//=========================================================================
	//	FUNCTION:	Get
	//	
	//	VISIBLITY:	Public
	//=========================================================================
	template <EventParamName NAME, EventParamType TYPE>
	bool CP_CarbonEvent_Param<NAME, TYPE>::Get( EventRef inEvent, ValueType& value,
		const std::nothrow_t& ) throw()
	{
		bool				result = false;
		OSStatus			status = CP_noErr;
		ValueType			currentValue = value;
		
		status = IOType::Get( inEvent, NAME, value );
		
		if( status == CP_noErr )
		{
			result = true;
		}
		else
		{
			value = currentValue;
			result = false;
		}
		
		return( result );
	}

	//=========================================================================
	//	FUNCTION:	Set
	//	
	//	VISIBLITY:	Public
	//=========================================================================
	template <EventParamName NAME, EventParamType TYPE>
	void CP_CarbonEvent_Param<NAME, TYPE>::Set( EventRef inEvent, const ValueType& inValue)
	{
		OSStatus			status = CP_noErr;
		
		status = IOType::Set( inEvent, NAME, inValue );
		CP_ThrowIfOSErr(status);
	}
#endif

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_CarbonEvent_Params

