//=========================================================================
//
//	FILE:				CP_HIView_ViewImp.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CPLAT implementatio of HIView classes that interface
//						with the CP_HIView class and the CPLAT framework to provide
//						the base HIView support.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 251 $
//						Latest Checkout:  $Date: 2005-08-22 08:58:47 -0700 (Mon, 22 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_HIView_ViewImp
#define _H_CP_HIView_ViewImp
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC && TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_HIView.h"
#include "CP_Point.h"
#include "CP_Size.h"
#include "CP_View_Imp_Base.h"
#include "CP_OSStatus.h"
#include "CP_Region.h"
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_CC_METROWERKS
#pragma mark === CP_HIView_ViewImp ==
#endif

/*!
*	@class		CP_HIView_ViewImp
*	@brief		CP_HIView class that provides the implementation and interface between HIView and CPLAT.
*
*	@ingroup	UI
*/
class CP_HIView_ViewImp : public CP_HIView
{
public:
	//! @name Initialization
	//@{
	static ::OSStatus					Create(
											HIViewRef *outControl,
											const HIRect *inBounds = NULL,
											WindowRef inWindow = NULL );
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_HIView_ViewImp( HIViewRef inControl );
	//! Destructor.
	virtual								~CP_HIView_ViewImp() throw();
	//@}
	
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Getters
	//@{
		virtual ControlKind				GetKind();
		virtual ::OSStatus				GetRegion(
												::ControlPartCode inPart,
												RgnHandle outRgn );
	//@}
	
	//! @name Testing
	//@{
		virtual ControlPartCode			HitTest( const HIPoint& inWhere );
	//@}
		
private:
	//! @name Initialization
	//@{
		static ::OSStatus				Construct(
												HIObjectRef inObjectRef,
												CP_HIObject **ouCP_HIObject );
	//@}

	//! @name Events
	//@{
											// Drawing
			void						HandleDrawEvent( CP_Event_Base& inEvent );

											// Keyboard
			void						HandleGetFocusPartEvent( CP_Event_Base& inEvent );
			void						HandleSetFocusPartEvent( CP_Event_Base& inEvent );

											// Scrolling
			void						HandleScrollInfoEvent( CP_Event_Base& inEvent );
			void						HandleScrollToEvent( CP_Event_Base& inEvent );
			
											// Initialize
			void 						HandleInitializeEvent( CP_Event_Base& inEvent );

											// Drag and Drop
			void 						HandleDragEnterEvent( CP_Event_Base& inEvent );
			void 						HandleDragLeaveEvent( CP_Event_Base& inEvent );
			void 						HandleDragReceiveEvent( CP_Event_Base& inEvent );

			void 						HandleHitTestEvent( CP_Event_Base& inEvent );
			void						HandleTrackEvent( CP_Event_Base& inEvent );

											// Mouse
			void						HandleMouseMovedEvent( CP_Event_Base& inEvent );
			void						HandleMouseEnteredEvent( CPLAT::CP_Event_Base& inEvent );
			void						HandleMouseExitedEvent( CPLAT::CP_Event_Base& inEvent );
	//@}

	//! @name Registration
	//@{
		static void 					RegisterClass();
	//@}

	// class data
protected:

	MouseTrackingRef					fMouseTrackingRef;
};

CPLAT_End_Namespace_CPLAT

#endif	//	TARGET_OS_MAC && TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2

#endif	// _H_CP_HIView_ViewImp

