//=========================================================================
//
//	FILE:				CP_View_Imp_MLTE.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 228 $
//						Latest Checkout:  $Date: 2005-07-04 16:35:04 -0700 (Mon, 04 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_View_Imp_MLTE
#define _H_CP_View_Imp_MLTE
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_KeyInputFilters.h"
#include "CP_MLTESupport.h"
#include "CP_View_Imp_Control.h"

CPLAT_Begin_Namespace_CPLAT

typedef struct
{
	RGBColor		sForground;
	RGBColor		sBackground;
	PenState		sPen;
} STPPaneState;

class CP_EXPORT CP_View_Imp_MLTE : public CP_View_Imp_Control, public CP_Object
{
public:
											// Construction / destruction
										CP_View_Imp_MLTE(
												CP_View_Imp *inOwnerViewImp,
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inHasHorizontalScroll = true,
												bool inHasVerticalScroll = true,
												bool inHasGrowBox = true,
												bool inReadOnly = false,
												bool inDrawFocusBorder = true );
	virtual								~CP_View_Imp_MLTE() throw();
			

	//! @name Appearance
	//@{
			void						Activate();
			void						Deactivate();
	//@}

	//! @name Command Handling
	//@{
			void						HandleCommandStatus(
												CP_CommandStatus& inCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommandNo );
										
			void						HandleCutCommand( OSStatus& inOutStatus );
			void						HandleCopyCommand( OSStatus& inOutStatus );
			void						HandleClearCommand( OSStatus& inOutStatus );
			void						HandlePasteCommand( OSStatus& inOutStatus );
			void						HandleSelectAllCommand( OSStatus& inOutStatus );
			void						HandleUndoCommand( OSStatus& inOutStatus );
			void						HandleRedoCommand( OSStatus& inOutStatus );
	//@}

	//! @name Drawing
	//@{
	virtual	void						HandleDraw( ControlRef inControl );
	virtual	void						PrepareDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );
	//@}

	//! @name Events
	//@{
			OSStatus					HandleCarbonEvent(
												EventHandlerCallRef inHandler,
												EventRef inEvent );

			OSStatus					HandleMouseDown( CP_Event_Base& inEvent );

	virtual	ControlPartCode				HandleHitTest(
												ControlRef inControl,
												const CP_Point& inMouseLocation );
	virtual	ControlPartCode				HandleFocus(
												ControlRef inControl,
												ControlFocusPart inAction );
	virtual	void						HandleIdle( ControlRef inControl );
	//@}

	//! @name Focus
	//@{
	virtual	void						OnLosingFocus();
	//@}

	//! @name Getters
	//@{
			OSStatus					ReadTextFromFile( const CP_FileSystemSpec& inFileSpec );

			CP_String					GetText( char inLineEnd = CP_LF );
			UInt32 						GetTextLength();
	//@}

	//! @name Key Filter
	//@{
			void						AttachFilter( CP_KeyInputFilter *inFilter );
			CP_KeyInputFilter			*GetFilter();
	//@}

	//! @name Positioning
	//@{
	virtual	void						SetPortBounds( const CP_Rect& inBounds );
	//@}

	//! @name Printing
	//@{
			void						Print();
	//@}

	//! @name Selection
	//@{
			void						GetSelection(
												UInt32 *inStartSelection,
												UInt32 *inEndSelection );
			void						SetSelection(
												UInt32 inStartSelection,
												UInt32 inEndSelection );
			bool						HasSelection();
	//@}

	//! @name Setters
	//@{
			void						SetText( const CP_String& inText );
			void						SetText( CP_InStream *inStream );
			void						InsertText( const CP_String& inText );
			void						InsertText(
												const char *inTextPtr,
												SInt32 inTextLength );
			void						SetFontSpec( CP_FontSpec& inFontSpec );
	//@}

protected:
	//! @name Events
	//@{
	virtual	bool						HandleFieldExitChar( char inChar );
	//@}

	//! @name Misc
	//@{
			OSStatus					OpenControl(
												ControlHandle inControl,
												bool inHasHorizontalScroll = true,
												bool inHasVerticalScroll = true,
												bool inHasGrowBox = true,
												bool inReadOnly = false );
			OSStatus					CloseControl( ControlHandle inControl );
	//@}

	//! @name Tracking
	//@{
	virtual	ControlPartCode				HandleTracking(
												ControlRef inControl,
												const CP_Point& inMouseLocation,
												ControlActionUPP actionProc );
	//@}

private:
	//! @name Drawing
	//@{
			void						TPPaneDrawEntry( STPPaneState *ps );
			void 						TPPaneDrawExit( STPPaneState *ps );
	//@}

	//! @name Events
	//@{
			void						CommandUpdateStatus( CP_Event_Base& inEvent );
			void						CommandProcess( CP_Event_Base& inEvent );

			void						HandleTextInputEvents( CP_Event_Base& inEvent );

	static pascal ::OSStatus			CarbonEventHandler(
												EventHandlerCallRef inHandler,
												EventRef inEvent,
												void *inUserData );
	//@}

	//! @name Misc
	//@{
			void 						TextChanged();
	//@}

	// class data
protected:
	CP_MLTESupport						fMLTESupport;

	CGrafPtr							fDrawingEnvironment;
	bool								fInDialogWindow;
	EventHandlerUPP						fHandlerUPP;
	EventHandlerRef						fHandlerRef;

	CP_KeyInputFilter					*fKeyFilter;			// object used to filter our keystrokes

		// signals
	CP_DataValueSignal					fFieldExitSignal;		/*! Signal sent when the user enters a field exit key. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_View_Imp_MLTE

