//=========================================================================
//
//	FILE:				CP_CheckBox.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms check box control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_CheckBox
#define _H_CP_CheckBox
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_CheckBox
*	@brief		Implementation of a platforms check box control.
*
*	@section 	persistance Persistance:
*				The following tags are implemented to create the button from
*				a persistance stream:
*					- kCP_Persist_ControlSignature - UInt32
*					- kCP_Persist_Bounds - CP_Rect
*					- kCP_Persist_Text - CP_String
*					- kCP_Persist_Command - UInt32
*					- kCP_Persist_IsHidden - bool
*					- kCP_Persist_IsEnabled - bool
*					- kCP_Persist_InitialState - SInt32
*					- kCP_Persist_AutoToggle - bool
*
*	@section 	signals Signals and Slots:
*				The following signals and slot are implemented in this class:
*					- None
*
*				From CP_PlatformControl:
*					- kCP_Signal_Clicked
*					- kCP_Signal_ValueChanged
*
*	@ingroup	UI
*/
class CP_EXPORT CP_CheckBox : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_CheckBox();
										CP_CheckBox(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												UInt32 inCommand,
												SInt32 inInitialValue = 0,
												bool inAutoToggle = true );
	//! Destructor.
	virtual								~CP_CheckBox() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
protected:
	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												UInt32 inCommand,
												SInt32 inInitialValue,
												bool inAutoToggle );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Value
	//@{
	virtual	void						SetValue( SInt32 inValue );
	//@}

	//! @name Slots
	//@{
	virtual	void						HandleCommandProcess( CP_Event_Base& inEvent );
	//@}

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												UInt32 inCommand,
												SInt32 inInitialValue,
												bool inAutoToggle );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												UInt32 inCommand,
												SInt32 inInitialValue,
												bool inAutoToggle );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_CheckBox
