//=========================================================================
//
//	FILE:				CP_Clock.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms clock control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Clock
#define _H_CP_Clock
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Clock
*	@brief		Implementation of a platforms clock control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_Clock : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Clock();
								CP_Clock(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize,
										UInt16 inClockType = 0,
										UInt32 inClockFlags = 0 );
	//! Destructor.
	virtual						~CP_Clock() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist			*PersistCreator();
	//@}
			
protected:
	//! @name Persistance
	//@{
	virtual	void				OnInitPersist(
										CP_View *inSuperview,
										CP_DataValueList& inDataValueList );
	//@}

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void				CreatePlatformImp(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize,
										UInt16 inClockType = 0,
										UInt32 inClockFlags = 0 );
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Initialization
	//@{
			void				CreatePlatformImp(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize,
										UInt16 inClockType = 0,
										UInt32 inClockFlags = 0 );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void				CreatePlatformImp(
										CP_View *inSuperview,
										const CP_Point& inUpperLeft,
										const CP_Size& inContentSize,
										UInt16 inClockType = 0,
										UInt32 inClockFlags = 0 );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Clock
