//=========================================================================
//
//	FILE:				CP_TV_DataBrowser_Defines.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Definitions for CP_TableView.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 244 $
//						Latest Checkout:  $Date: 2005-07-26 18:28:15 -0700 (Tue, 26 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_TV_DataBrowser_Defines
#define _H_CP_TV_DataBrowser_Defines
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueHolder.h"
#include "CP_Types.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

	// forward declarations
class CP_TV_DataBrowser;
class CP_TV_DataBrowserRow;
class CP_TV_DataBrowser_CH_Base;

//=========================================================================
//	Content Type
//=========================================================================
#if TARGET_OS_LINUX
#else
	const FourCharCode kCP_Content_IconAndText					= CP_FOUR_CHAR_CODE('ICTX');
	const FourCharCode kCP_Content_String						= CP_FOUR_CHAR_CODE('STRN');
	const FourCharCode kCP_Content_EditString					= CP_FOUR_CHAR_CODE('EDST');
	const FourCharCode kCP_Content_View							= CP_FOUR_CHAR_CODE('EDVW');
#endif

//=========================================================================
//	Content View Types
//=========================================================================
enum
{
	kCP_TV_DB_ViewType_CheckBox									= CP_FOUR_CHAR_CODE('CHCK'),
	kCP_TV_DB_ViewType_Popup									= CP_FOUR_CHAR_CODE('POPU'),
	kCP_TV_DB_ViewType_Progress									= CP_FOUR_CHAR_CODE('PROG'),
	kCP_TV_DB_ViewType_PushButton								= CP_FOUR_CHAR_CODE('PUSH'),
	kCP_TV_DB_ViewType_RadioButton								= CP_FOUR_CHAR_CODE('RADB'),
	kCP_TV_DB_ViewType_Slider									= CP_FOUR_CHAR_CODE('SLID'),
	kCP_TV_DB_ViewType_UserView									= CP_FOUR_CHAR_CODE('USVW')
};

//=========================================================================
//	Enums
//=========================================================================
enum
{
	kCP_TV_DB_HasChildren						= 0x0001,		// row has a disclosure indicator
	kCP_TV_DB_IsExpanded						= 0x0002		// row has children and is expanded
};

//=========================================================================
//	Misc Defines
//=========================================================================
const SInt32 kCP_TV_DB_InvalidRowIndex 								= -1;
const SInt16 kCP_TV_DB_DisclosureImageSize							= 16;
const SInt16 kCP_TV_DB_DefaultRowIndent								= 16;

//=========================================================================
//	Structures
//=========================================================================
	// structure to maintain information on the properties of our DataBrowser.
struct CP_S_TV_DB_Properties
{
	SInt32					fNumberRows;				// number of rows
	SInt32					fNumberColumns;				// number of Column
	SInt32					fRowHeight;					// default row height
	SInt32					fColumnWidth;				// default column width

	UInt32					fGeometryType;				// rows and column same size

	UInt32					fAllowMultiSelection;		// allow multi selection of rows
	
							CP_S_TV_DB_Properties(
									SInt32 inColumns = 1,
									SInt32 inRows = 0,
									SInt32 inColumnWidth = 50,
									SInt32 inRowHeight = 20 );
							CP_S_TV_DB_Properties( const CP_S_TV_DB_Properties& inProperties );

	CP_S_TV_DB_Properties&	operator =( const CP_S_TV_DB_Properties& inProperties );
};

struct CP_S_TV_DB_CellData
{
	FourCharCode				fContentType;
	SInt32						fCellID;
	CP_View						*fHelperView;					// a helper view associated with the cell		
	CP_Rect						fHelperViewInsets;				// amount to inset helper views bounds from cell bounds

								CP_S_TV_DB_CellData();
								CP_S_TV_DB_CellData( const CP_S_TV_DB_CellData& inCellData );

	CP_S_TV_DB_CellData&		operator =( const CP_S_TV_DB_CellData& inCellData );
};

struct CP_S_TV_DB_ColumnParams
{
	FourCharCode				fColumnContentType;					// content type for the Columnumn
	SInt32						fColumnWidth;						// width of the Columnumn
	SInt32						fColumnOffset;						// offset from left edge to begining of cell drawing
	SInt32						fColumnMinSize;						// minimum size this Columnumn can become
	CP_DataValueList			fColumnProperties;					// Content handler properties for this column
	
								CP_S_TV_DB_ColumnParams();
								CP_S_TV_DB_ColumnParams( const CP_S_TV_DB_ColumnParams& inColumnParams );

	CP_S_TV_DB_ColumnParams&	operator =( const CP_S_TV_DB_ColumnParams& inColumnParams );
};

	// Structure used to describe a content handler 
struct CP_S_TV_DB_ContentHandlerInfo
{
	FourCharCode				fContentType;
	CP_TV_DataBrowser_CH_Base	*fContentHandler;

								CP_S_TV_DB_ContentHandlerInfo();
								CP_S_TV_DB_ContentHandlerInfo(
										FourCharCode inContentType,
										CP_TV_DataBrowser_CH_Base *inContentHandler );
								CP_S_TV_DB_ContentHandlerInfo( const CP_S_TV_DB_ContentHandlerInfo& inInfo );

	CP_S_TV_DB_ContentHandlerInfo&	operator =( const CP_S_TV_DB_ContentHandlerInfo& inInfo );
};

struct CP_S_TV_DB_ParamBlock
{
	CP_Rect						fCellBounds;					// the bounnds of the cell
	CP_Point					fCell;
	SInt32						fCellID;						// Data source id of the cell
	CP_DataValueHolder			fCellData;
	SInt32						fCellIndent;					// amount to indent our drawing
	bool						fCellIsSelected;				// draw the cell in a selected state
	FourCharCode				fCellContentHandler;			// the content handler for this cell
	CP_TV_DataBrowser			*fOwnerDataBrowser;				// data browser this data is used in		
	CP_View						*fHelperView;					// a helper view associated with the content		
	CP_Rect						fHelperViewInsets;				// amount to inset helper views bounds from cell bounds
	SInt32						fUserData;						// user data for the content handler
	
								CP_S_TV_DB_ParamBlock();
								CP_S_TV_DB_ParamBlock( const CP_S_TV_DB_ParamBlock& inParamBlock );

	CP_S_TV_DB_ParamBlock&		operator =( const CP_S_TV_DB_ParamBlock& inParamBlock );
};

struct CP_S_TV_DB_DataSourceView
{
	SInt32						fViewType;
	SInt32						fValue;
	SInt32						fMinimum;
	SInt32						fMaximum;
	CP_String					fTitle;
	UInt32 						fCommand;
	CP_Rect						fHelperViewInsets;				// amount to inset helper views bounds from cell bounds
	CP_Menu						*fHelperMenu;
	
								CP_S_TV_DB_DataSourceView();
								CP_S_TV_DB_DataSourceView(
										SInt32 inViewType,
										SInt32 inValue,
										SInt32 inMinimum,
										SInt32 inMaximum,
										const CP_String& inTitle,
										UInt32 inCommand );
								CP_S_TV_DB_DataSourceView( const CP_S_TV_DB_DataSourceView& inInfo );

	CP_S_TV_DB_DataSourceView&	operator =( const CP_S_TV_DB_DataSourceView& inInfo );
};

struct CP_S_TV_DB_SortInfo
{
	CP_TV_DataBrowserRow				*fDataBrowserCellDataL;
	CP_TV_DataBrowserRow				*fDataBrowserCellDataR;
};

//=========================================================================
//	Typedefs
//=========================================================================
	// stores our various contennt handlers used to handle the DataBrowsers data
typedef std::vector<CP_S_TV_DB_ContentHandlerInfo>						CP_ContentHandlers_Array;
typedef std::vector<CP_S_TV_DB_ContentHandlerInfo>::iterator			CP_ContentHandlers_Iterator;

	// stores information about each Columnumn in the DataBrowser
typedef std::vector<CP_S_TV_DB_ColumnParams>							CP_DataBrowserColumn_Array;
typedef std::vector<CP_S_TV_DB_ColumnParams>::iterator					CP_DataBrowserColumn_Iterator;

	// stores information about the cell data for each cell in a row
typedef std::vector<CP_S_TV_DB_CellData>								CP_DataBrowserCellData_Array;
typedef std::vector<CP_S_TV_DB_CellData>::iterator						CP_DataBrowserCellData_Iterator;

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TV_DataBrowser_Defines

