//=========================================================================
//
//	FILE:				CP_WindowManager.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A singleton class to manage windows in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 256 $
//						Latest Checkout:  $Date: 2005-08-26 19:35:37 -0700 (Fri, 26 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_WindowManager
#define _H_CP_WindowManager
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_CommandHandler.h"
#include "CP_NoCopy.h"
#include "CP_Window.h"


#include "CP_SignalSlot.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_LINUX
	typedef std::map<OSWindow, CP_Window_Imp *, std::less<OSWindow> >							CP_Window_Map;
	typedef std::map<OSWindow, CP_Window_Imp *, std::less<OSWindow> >::iterator					CP_Window_Iterator;

	typedef std::pair<const OSWindow, CP_Window_Imp *>											CP_Window_Entry;

	typedef std::list<CP_Window_Imp *>															CP_WindowLayer_List;
	typedef std::list<CP_Window_Imp *>::iterator												CP_WindowLayer_Iterator;
	typedef std::list<CP_Window_Imp *>::reverse_iterator										CP_WindowLayer_ReverseIterator;
#else
	typedef std::map<OSWindow, CP_Window_Imp *, std::less<OSWindow> >							CP_Window_Map;
	typedef std::map<OSWindow, CP_Window_Imp *, std::less<OSWindow> >::iterator					CP_Window_Iterator;

	typedef std::pair<const OSWindow, CP_Window_Imp *>											CP_Window_Entry;

	#if TARGET_OS_WIN32
		typedef std::list<CP_Window_Imp *>														CP_WindowLayer_List;
		typedef std::list<CP_Window_Imp *>::iterator											CP_WindowLayer_Iterator;
		typedef std::list<CP_Window_Imp *>::reverse_iterator									CP_WindowLayer_ReverseIterator;
	#endif
#endif

/*!
*	@class		CP_WindowManager
*	@brief		A singleton class to manage windows in CPLAT.
*/
class CP_EXPORT CP_WindowManager : public CP_NoCopy, public CP_SigSlot::CP_SlotObject
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_WindowManager();
	//! Destructor.
	virtual								~CP_WindowManager() throw();
	//@}
			
	//! @name Instance
	//@{
	static	CP_WindowManager			*Instance();
	//@}

	//! @name Desktop
	//@{
			SInt16						GetDesktopWidth();
			SInt16						GetDesktopHeight();
			CP_Rect						GetAvailableWindowPositioningArea();
	//@}

	//! @name Misc
	//@{
			CP_Window_Imp 				*GetNamedWindow( const CP_String& inTitle );
	
			void						RefreshAll();
	//@}

	//! @name Positioning
	//@{
			void						Reposition(
												CP_Window *inWindow,
												CP_Window *inParent,
												SInt16 inMethod );

			SInt32						GetIndexOfWindow( CP_Window_Imp *inWindow );
	//@}

	//! @name Regular Windows
	//@{
			CP_Window_Imp 				*GetTopRegularWindow();
			CP_Window_Imp 				*GetNthRegularWindow( SInt16 inIndex );
	//@}

	//! @name Window Mapping
	//@{
			void						AddWindow(
												CP_Window_Imp *inCPWindow,
												OSWindow inOSWindow );
			CP_Window_Imp				*GetWindow( OSWindow inOSWindow );

			void						RemoveOSWindow( OSWindow inOSWindow );

			SInt16						GetNumberWindows();

			void						CloseAll();
	//@}

	//! @name Windows
	//@{
			CP_Window_Imp				*GetNthWindow( SInt16 inIndex );
	//@}

protected:

	// class data
protected:
	static CP_WindowManager				*fsInstance;		/*! Single instance of this class. */
	
	CP_Window_Map						*fWindowMap;

	#if TARGET_OS_WIN32 || TARGET_OS_LINUX
		CP_WindowLayer_List				*fRegularLayer;		/*! CP_Windows in the regular window layer */
	    							
		CP_WindowLayer_List				*fModalLayer;		/*! CP_Windows in the modal window layer */
	    							
		CP_WindowLayer_List				*fFloatingLayer;	/*! CP_Windows in the floating window layer */

		CP_Point						fStackOffset;		/*! Amount to offset each when when stacking. */

		CP_Point						fNextStackPosn;		/*! The next position to stack a window at */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
public:
			void						EnableWindowsForDialog(
												CP_Window *inDialogWindow,
												bool inEnable );
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_WindowManager

