//=========================================================================
//
//	FILE:				CP_Graphics_Imp_GDIPlus.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Windows GDI+ implementation of the CP_Graphics object.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 247 $
//						Latest Checkout:  $Date: 2005-07-30 19:06:38 -0700 (Sat, 30 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Graphics_Imp_GDIPlus
#define _H_CP_Graphics_Imp_GDIPlus
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_WIN32 && CP_Optional_GDIPlus

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_Graphics_Imp.h"
#include "CP_Types.h"

	// Windows SDK
#include "GdiPlus.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Graphics_Imp_GDIPlus
*	@brief		CP_Graphics_Imp subclass to perform drawing using Windows GDIPlus.
*
*				This is the CP_Graphics_Imp class used by a CP_Graphics object when the
*				user has selected GDIPlus as the underlying graphics engine. This will
*				also be selected as the native platform type when running under Windows
*				and GDIPlus drawing is available.
*/
class CP_EXPORT CP_Graphics_Imp_GDIPlus : public CP_Graphics_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Graphics_Imp_GDIPlus( CP_Canvas& inDestCanvas );
	//! Destructor.
	virtual								~CP_Graphics_Imp_GDIPlus();
	//@}

	//! @name Setters
	//@{
	virtual	void						SetAlpha( float inAlpha );
	virtual	void						SetLineWidth( float lineWidth );
	virtual	void						SetLineJoin( CP_E_GraphicsLineJoin inLineJoin );
	virtual	void						SetLineCap( CP_E_GraphicsLineCap inLineCap );
	virtual	void						SetStrokeColor( CP_ARGBColor& inStrokeColor );
	virtual void						SetFillColor( CP_ARGBColor& inFillColor );
	//@}
			
	//! @name Gradient
	//@{
	virtual	void						DrawGradient(
												CP_E_GradientType inGradientType,
												CP_Rect& inDestBounds );
	virtual	void						SetGradientAngle( SInt16 inAngle )						{ fGradientAngle = inAngle; }
	virtual	void						SetGradientColor(
												const SInt16 inIndex,
												const CP_ARGBColor& inColour );
	virtual	void						SetGradientCount( SInt16 inGradientCount )				{ fGradientCount = inGradientCount; }
	//@}

	//! @name Graphics State
	//@{
	virtual	void						SaveState();
	virtual	void						RestoreState();

	virtual	void						ScaleCTM(
												float inHorAmt,
												float inVerAmt );
	virtual	void						RotateCTM( float inAngle );
	//@}

	//! @name Bezier
	//@{
	virtual	void						StrokeBezier(
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inControlPointTwo,
												const CP_Point& inEndPoint );
	//@}

	//! @name Icons
	//@{
			void						DrawIcon(
												const CP_Rect& inBounds,
												CP_E_IconStyle inIconType,
												SInt16 inIconID );
			void						DrawIcon(
												CP_OSIcon& inPicture,
												const CP_Rect& inBounds );
	//@}

	//! @name Images
	//@{
	virtual	void						DrawImage(
												const CP_FileSystemSpec& inImageSpec,
												CP_Rect& inDestBounds );
	virtual	void						DrawImage(
												const CP_FileSystemSpec& inImageSpec,
												CP_Rect& inSourceBounds,
												CP_Rect& inDestBounds );
	virtual	void						DrawPicture(
												CP_OSPicture& inPicture,
												const CP_Rect& inBounds );
	//@}

	//! @name Lines
	//@{
	virtual	void						Line(
												const CP_Point& inPoint,
												const CP_Size& inSize );
	virtual	void						Line(
												float inStartHorLocation,
												float inStartVerLocation,
												float inEndHorLocation,
												float inEndVerLocation );
	//@}

	//! @name Paths
	//@{
	virtual	void						BeginPath();
	virtual	void						ClosePath();
	virtual	void						StrokePath();
	virtual	void						FillPath();
	virtual	void						PathMoveTo( const CP_Point& inPoint );
	virtual	void						PathMoveTo(
												float inHorLocation,
												float inVerLocation );
	virtual	void						PathAddLineTo( const CP_Point& inPoint );
	virtual	void						PathAddLines(
												const CP_Point inPoint[],
												SInt16 inNumberPoints );
	virtual	void						PathAddLine(
												const CP_Point& inStartPoint,
												const CP_Point& inEndPoint );
	virtual	void						PathAddLine(
												float inStartH,
												float inStartV,
												float inEndH,
												float inEndV );
	virtual	void						PathAddRect( const CP_Rect& inBounds );
	virtual	void						PathAddOval( const CP_Rect& inBounds );
	virtual	void						PathAddArc(
												float x,
												float y,
												float width, 
												float height,
												float startAngle,
												float sweepAngle );
	virtual	void						PathAddArc(
												float x,
												float y,
												float radius,
												float startAngle,
												float endAngle,
												SInt16 clockwise );
		virtual	void					PathAddPoly( CP_Point_Array& inPoints );
	virtual	void						ClipPath();
	//@}

	//! @name Polygons
	//@{
	virtual	void						StrokePoly( CP_Point_Array& inPoints );
	virtual	void						FillPoly( CP_Point_Array& inPoints );
	//@}

	//! @name Rects
	//@{
	virtual	void						StrokeRect( const CP_Rect& inBounds );
	virtual	void						FillRect( const CP_Rect& inBounds );
	virtual	void						EraseRect( const CP_Rect& inBounds );
	//@}
			
	//! @name Round Rects
	//@{
	virtual	void						StrokeRoundRect(
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
	virtual	void						FillRoundRect( 
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
	//@}

	//! @name Oval
	//@{
	virtual	void						StrokeOval( const CP_Rect& inBounds );
	virtual	void						FillOval( const CP_Rect& inBounds );
	//@}

	//! @name Text
	//@{
	virtual	void						SetTextRenderingFlags( SInt16 inFlags );
	virtual	void						DrawString(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Point& inStartPoint );
	virtual	void						DrawText(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Rect& inBounds,
												SInt16 inHorAlign = CP_E_TextAlign_HorLeft,
												SInt16 inVerAlign = CP_E_TextAlign_VerTop,
												bool inWrapText = true );
	virtual	void						DrawTrunc(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Rect& inBounds,
												SInt16 inHorAlign = CP_E_TextAlign_HorLeft,
												SInt16 inVerAlign = CP_E_TextAlign_VerTop );
	//@}

protected:

private:
	//! @name Gradient
	//@{
			void						DrawGradient_imp(
												CP_E_GradientType inGradietType,
												CP_Rect& inDestBounds );

	virtual	void						BuildGradient(
												CP_Rect& inDestBounds,
												CP_Point& inCenterPoint );

			void						InitGradient();

			CP_E_GradientType			GetGradiientType()										{ return( fGradientType ); }
			SInt16						GetGradientAngle()										{ return( fGradientAngle ); }
			SInt16						GetNumberGradients()									{ return( fGradientCount ); }

			SInt32						CalcGradientRange(
												CP_Rect& inBounds,
												CP_Point& inCenterPoint );
			void						CalcGradientColors(
												SInt32 inRange,
												CP_RGBColor& inOutColorOne,
												CP_RGBColor& inOutColorTwo,
												CP_E_GradientColorInfo *inOutGradientInfo );	
			void						AdjustColor(
												CP_RGBColor& inOutColor,
												CP_E_GradientColorInfo *inOutGradientInfo );
			void						CalcEnclosingBounds(
												CP_Rect& inOutBounds,
												CP_Point& inCenterPoint );	

			void						BuildRect(
												CP_Rect& inDestBounds,
												CP_Point& inCenterPoint,
												CP_E_GradientType inGradientType );
			void						BuildRadial(
												CP_Rect& inBounds,
												CP_Point& inCenterPoint );
			void						BuildSunburst(
												CP_Rect& inBounds,
												CP_Point& inCenterPoint,
												CP_E_GradientType inGradientType );
			void						BuildShapeburst(
												CP_Rect& inBounds,
												CP_Point& inCenterPoint );
			void						BuildAngled( CP_Rect& inBounds );

			UInt16						CP_FixedToColor( CP_Fixed inValue );
	//@}

	// class data
protected:
	Gdiplus::Graphics					*fGraphicsContext;		/*! Gdi+ graphics context. */
	Gdiplus::Pen						*fGraphicsPen;			/*! Gdi+ graphics pen. */
	Gdiplus::GraphicsPath				*fPath;					/*! Gdi+ graphics path. */

	Gdiplus::GraphicsState				fGraphicState;			/*! Gdi+ graphics state. */

	CP_E_GradientType					fGradientType;			/*! Type of gradient. */
	SInt16								fGradientCount;			/*! Number of colors in gradient. */
	CP_ARGBColor						fColors[ kCP_MaxGradientColors ];		/*! List of colors in gradient. */
	SInt16								fOffsets[ kCP_MaxGradientColors ];		/*! Offsets used to draw gradient. */
	SInt16								fGradientAngle;			/*! Angle used to draw gradient. */
};

CPLAT_End_Namespace_CPLAT

#endif	// #if TARGET_OS_WIN32 && CP_Optional_GDIPlus

#endif	// _H_CP_Graphics_Imp_GDIPlus
